# Experiments
INTERVAL = "PT2M"
MAX_RUNS = 10
RESULTS_PATH = "results/data/"
REPEAT_INTERVAL = "PT3M"
THROUGHPUT_JSON_FORMAT = "throughput_%n.json"
THROUGHPUT_JSON_FORMAT_WILDCARD = "throughput_*.json"
RTT_JSON_FORMAT = "rtt_%n.json"
RTT_JSON_FORMAT_WILDCARD = "rtt_*.json"
LATENCY_JSON_FORMAT = "latency_%n.json"
LATENCY_JSON_FORMAT_WILDCARD = "latency_*.json"
PERFSONAR_JSON_OUTPUT_FORMAT = "json"
THROUGHPUT_EXPERIMENT_NAME = "Throughput"
RTT_EXPERIMENT_NAME = "Round Trip Time"
LATENCY_EXPERIMENT_NAME = "Latency"
WIRELESS = "Wireless"
WIRED = "Wired"
THROUGHPUT_UNIT_OF_MEASURE="MBps"
RTT_UNIT_OF_MEASURE="ms"
LATENCY_UNIT_OF_MEASURE="ms"



# Wired
EMU_EMU_WIRED_PREFIX = "wired_emu_emu_"
EMU_PHY_WIRED_PREFIX = "wired_emu_phy_"
PHY_PHY_WIRED_PREFIX = "wired_phy_phy_"

## Emu Emu
PERFSONAR_TESTPOINT_UBUNTU = "alexandremitsurukaihara/lft:perfsonar-testpoint-ubuntu"
USR_SBIN_INIT_COMMAND = "/usr/sbin/init"
EMU_EMU_WIRED_H1_IP = "10.0.0.1"
EMU_EMU_WIRED_H2_IP = "10.0.0.2"

EMU_EMU_WIRED_HOST_IP = "192.0.0.1" 


## Emu Phy
EMU_PHY_WIRED_H1_IP = "10.0.0.2"
EMU_PHY_WIRED_H2_IP = "38.68.234.53"


## Phy Phy
PHY_PHY_WIRED_H1_IP = "38.68.234.50"
PHY_PHY_WIRED_H2_IP = "38.68.234.53"


# Wireless
EMU_EMU_WIRELESS_PREFIX = "wireless_emu_emu_"
EMU_PHY_WIRELESS_PREFIX = "wireless_emu_phy_"
PHY_PHY_WIRELESS_PREFIX = "wireless_phy_phy_"
SRSRAN_PERFSONAR_UHD_IMAGE = "alexandremitsurukaihara/lft:srsran-perfsonar-uhd4"

## Experiments
THROUGHPUT = "Throughput"
RTT = "RTT"
LATENCY = "Latency"
JITTER = "Jitter"

## Emu Emu
EMU_EMU_WIRELESS_EPC_IP_ADDR = "172.16.0.1"
EMU_EMU_WIRELESS_UE_IP_ADDR = "172.16.0.2"

EMU_PHY_WIRELESS_EPC_IP_ADDR = "172.16.0.1"
EMU_PHY_WIRELESS_UE_IP_ADDR = "10.0.0.1"
EMU_PHY_WIRELESS_USRP_IP_ADDR = "192.168.107.2"

PHY_PHY_WIRELESS_EPC_IP_ADDR = "172.16.0.1"
PHY_PHY_WIRELESS_UE_IP_ADDR = "172.16.0.2"

# Plot Colors
EMU_EMU_PLOT_COLOR = 'red'
EMU_PHY_PLOT_COLOR = 'green'
PHY_PHY_PLOT_COLOR = 'blue'



# Docker commands 
DOCKER_COMMAND = "docker"
DOCKER_RUN = DOCKER_COMMAND + " run"

# Docker run options
NETWORK = "--network"
NAME = "--name"
PRIVILEGED = "--privileged"
DNS = "--dns"
MEMORY = "--memory"
CPUS = "--cpus"

# UE config file
RF_SECTION = "rf"
DEVICE_ARGS_ATTR = "device_args"
DEVICE_NAME_ATTR = "device_name" 
TX_GAIN_ATTR = "tx_gain"
RX_GAIN_ATTR = "rx_gain"
USIM_SECTION = "usim" 
ALGORITHM_ATTR = "algo"
IMSI_ATTR = "imsi"

## PHY section
PHY_SECTION = "phy"
CORRECT_SYNC_ERROR = "correct_sync_error"

# EPC config file
MME_SECTION = "mme"
MME_BIND_ADDR = "mme_bind_addr"

SPGW_SECTION = "spgw"
GTPU_BIND_ADDR = "gtpu_bind_addr"
SGI_IF_ADDR = "sgi_if_addr"

# eNB config file
ENB_SECTION = "enb"
MME_ADDR = "mme_addr"
GTP_BIND_ADDR = "gtp_bind_addr"
S1C_BIND_ADDR = "s1c_bind_addr"


# Copyright (C) 2022 Alexandre Mitsuru Kaihara
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import logging
import subprocess
from .node import Node


class Switch(Node): 
    # Brief: Instantiate a switch class, where it can be defined to capture flow data of each interface added
    # Params:
    # Return:
    #   None
    def __init__(self, name: str, hostPath='', containerPath=''):
        super().__init__(name)
        if hostPath == '' and containerPath == '':
            self.__mount = False
        elif hostPath != '' and containerPath != '':
            self.__hostPath = hostPath
            self.__containerPath = containerPath
            self.__mount = True
        else: 
            raise Exception(f"Invalid hostPath and containerPath mount point on {self.getNodeName()}. hostPath and containerPath cannot be null")

    # Brief: Instantiate an OpenvSwitch switch container
    # Params:
    # Return:
    #   None
    def instantiate(self, image='alexandremitsurukaihara/lst2.0:openvswitch', controllerIP='', controllerPort=-1) -> None:
        mount = ''
        if self.__mount: mount = f'-v {self.__hostPath}:{self.__containerPath}'
        super().instantiate(dockerCommand=f"docker run -d --network=none --privileged {mount} --name={self.getNodeName()} {image}")
        try:
            # Create bridge and set it up
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl add-br {self.getNodeName()}", shell=True)
            subprocess.run(f"docker exec {self.getNodeName()} ip link set {self.getNodeName()} up", shell=True)
        except Exception as ex:
            logging.error(f"Error while creating the switch {self.getNodeName()}: {str(ex)}")
            raise NodeInstantiationFailed(f"Error while creating the switch {self.getNodeName()}: {str(ex)}")
        # Link it to a controller
        if controllerIP != '' and controllerPort != -1:
            self.setController(controllerIP, controllerPort)

    # Brief: Set the controller to which the switch will be connecting to
    # Params:
    #   String ip: Controller's IP address
    #   String port: Controller's port
    # Return:
    #   None
    def setController(self, ip:str, port: int) -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl set-controller {self.getNodeName()} tcp:{ip}:{str(port)}", shell=True)
        except Exception as ex:
            logging.error(f"Error connecting switch {self.getNodeName()} to controller on IP {ip}/{port}: {str(ex)}")
            raise Exception(f"Error connecting switch {self.getNodeName()} to controller on IP {ip}/{port}: {str(ex)}")

    # Brief: Creates a port in OpenvSwitch bridge
    # Params:
    #   String nodeName: The name of the bridge is for default the same name of the switch container
    #   String peerName: Name of the interface to connect to the switch
    # Return:
    #   None
    def __createPort(self, nodeName, peerName) -> None:
        try:
            subprocess.run(f"docker exec {nodeName} ovs-vsctl add-port {nodeName} {peerName}", shell=True)
        except Exception as ex:
            logging.error(f"Error while creating port {peerName} in switch {nodeName}: {str(ex)}")
            raise Exception(f"Error while creating port {peerName} in switch {nodeName}: {str(ex)}")

    # Brief: Set Ip to an interface (the ip must be set only after connecting it to a container)
    # Params:
    #   String ip: IP address to be set to peerName interface
    #   int mask: Integer that represents the network mask
    #   String node: Reference to the node it is connected to this container to discover the intereface to set the ip to
    # Return:
    def setIp(self, ip: str, mask: int, interfaceName='') -> None:
        if interfaceName == '':
            interfaceName = self.getNodeName()
        self._Node__setIp(ip, mask, interfaceName)
    

    def enableNetflow(self, destIp: str, destPort: int, activeTimeout=60)  -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl -- set Bridge {self.getNodeName()} netflow=@nf --  --id=@nf create  NetFlow  targets=\\\"{destIp}:{destPort}\\\"  active-timeout={activeTimeout}", shell=True)
        except Exception as ex:
            logging.error(f"Error setting Netflow on {self.getNodeName()} switch: {str(ex)}")
            raise Exception(f"Error setting Netflow on {self.getNodeName()} switch: {str(ex)}")

    def clearNetflow(self) -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl clear Bridge {self.getNodeName()} netflow", shell=True)
        except Exception as ex:
            logging.error(f"Error clearing Netflow on {self.getNodeName()} switch: {str(ex)}")
            raise Exception(f"Error clearing Netflow on {self.getNodeName()} switch: {str(ex)}")

    def enablesFlow(self, destIp: str, destPort: int, header=128, sampling=64, polling=10)  -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl -- --id=@s create sFlow agent={self.getNodeName()} target=\\\"{destIp}:{destPort}\\\" header={str(header)} sampling={str(sampling)} polling={str(polling)} -- set Bridge {self.getNodeName()} sflow=@s", shell=True)
        except Exception as ex:
            logging.error(f"Error setting sFlow on {self.getNodeName()} switch: {str(ex)}")
            raise Exception(f"Error setting sFlow on {self.getNodeName()} switch: {str(ex)}")

    def clearsFlow(self) -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl clear Bridge {self.getNodeName()} sflow", shell=True)
        except Exception as ex:
            logging.error(f"Error clearing sFlow on {self.getNodeName()} switch: {str(ex)}")
            raise Exception(f"Error clearing sFlow on {self.getNodeName()} switch: {str(ex)}")

    def enableIPFIX(self, destIp: str, destPort: int, obsDomainId=123, obsPointId=456, cacheActiveTimeout=60, cacheMaxFlow=60, enableInputSampling=False, enableTunnelSampling=True) -> None:
        try:    
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl -- set Bridge {self.getNodeName()} ipfix=@i -- --id=@i create IPFIX targets=\\\"{destIp}:{destPort}\\\" obs_domain_id={str(obsDomainId)} obs_point_id={str(obsPointId)} cache_active_timeout={str(cacheActiveTimeout)} cache_max_flows={str(cacheMaxFlow)} other_config:enable-input-sampling={str(enableInputSampling).lower()} other_config:enable-tunnel-sampling={str(enableTunnelSampling).lower()}", shell=True)
        except Exception as ex:
            logging.error(f"Error setting IPFIX on {self.getNodeName()} switch: {str(ex)}")
            raise Exception(f"Error setting IPFIX on {self.getNodeName()} switch: {str(ex)}")

    def clearIPFIX(self) -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ovs-vsctl clear Bridge {self.getNodeName()} ipfix", shell=True)
        except Exception as ex:
            logging.error(f"Error clearing IPFIX on {self.getNodeName()} switch: {str(ex)}")
            raise Exception(f"Error clearing IPFIX on {self.getNodeName()} switch: {str(ex)}")

    # Brief: Set up the tshark to sniff all the packets into pcap files
    # Params:
    #   List<Node> nodes: References of the nodes connected to this switch to sniff packets
    #   boolean sniffAll: If sniff all is set  
    # Return:
    def collectFlows(self, nodes=[], path='', rotateInterval=60, sniffAll=False) -> None:
        try:
            interfaces = self._Node__getAllIntefaces()
            if sniffAll == False:
                if len(nodes) > 0: 
                    interfaces = [self._Node__getThisInterfaceName(node) for node in nodes]
                    interfaces.append(self.getNodeName())
                else:
                    raise Exception(f"Expected at least one node reference to sniff packets on {self.getNodeName()} switch")
            interfaces = list(set(interfaces) - set(['lo', 'ovs-system']))
            options = ['-i '+interface for interface in interfaces]
            options = ' '.join(options)
            subprocess.run(f"docker exec {self.getNodeName()} tshark {options} -b duration:{rotateInterval} -w {path}/dump.pcap > /dev/null 2>&1 &", shell=True)
        except Exception as ex:
            logging.error(f"Error set the collector on {self.getNodeName()}: {str(ex)}")
            raise Exception(f"Error set the collector on {self.getNodeName()}: {str(ex)}")

    # Brief: Set default route to forward all incoming packets to s1 bridge and let the bridge handle the forwarding
    # Params:
    # Return:
    def __addDefaultRoute(self) -> None:
        try:
            subprocess.run(f"docker exec {self.getNodeName()} ip route add 0.0.0.0/0 dev {self.getNodeName()}", shell=True)
        except Exception as ex:
            logging.error(f"Error adding route default route for switch {self.getNodeName()}: {str(ex)}")
            raise Exception(f"Error adding route default route for switch {self.getNodeName()}: {str(ex)}")



#!/usr/bin/env python
from gc import collect
import signal
import sys
import subprocess
from wsgiref.simple_server import ServerHandler, server_version
from host import Host
from switch import Switch
from controller import Controller
from configparser import ConfigParser
from node import Node
from os import getcwd
from globalvariables import *

class Seafile(Host):
    def updateServerConfig(self) -> None:
        self.copyContainerToLocal("/home/seafolder", "seafolder")
        out = subprocess.run("cat seafolder", shell=True, capture_output=True).stdout.decode('utf8')
        parser = ConfigParser()
        parser.read('serverconfig.ini')
        parser.set("50", "seafolder",  out)
        parser.set("200", "seafolder", out)
        parser.set("210", "seafolder", out)
        parser.set("220", "seafolder", out)
        with open('serverconfig.ini', 'w') as configfile:
            parser.write(configfile)


class LinuxClient(Host):
    def setAutomationScripts(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation")
    def setPrinterIp(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/system/printerip")
    def setSshIpList(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/system/sshiplist.ini")
    def setClientBehaviour(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/system/config.ini")
    def setServerConfig(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/system/serverconfig.ini")
    def setIpListPort80(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/attacking/ipListPort80.txt")
    def setIpList(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/attacking/ipList.txt")
    def setIpRange(self, path) -> None:
        self.copyLocalToContainer(path, "/home/debian/automation/packages/attacking/iprange.txt")


def setLinuxClientFileConfig(node: LinuxClient, subnet: str, behaviour: str):
    printTerm(f"[LST2.0] Copying Configuration Files to Container {node.getNodeName()}")
    if subnet != external_subnet: aux = "internal"
    else: aux = "external"
    node.setAutomationScripts("automation")
    node.setPrinterIp(f"printersip/{subnet.split('.')[2]}")
    node.setSshIpList("sshiplist.ini")
    node.setClientBehaviour(f"client_behaviour/{behaviour}.ini")
    node.setServerConfig("serverconfig.ini")
    node.setIpListPort80(f"{aux}_ipListPort80.txt")        
    node.setIpList(f"{aux}_ipList.txt")
    node.setIpRange(f"{aux}_iprange.txt")


def setNetworkConfig(node: Node, bridge: Node, subnet: str, address: int, setFiles=True) -> Node:
    node.connect(bridge)
    printTerm(f"[LST2.0] ... Connecting to {bridge.getNodeName()}")
    node.setIp(subnet+str(address), 24, bridge)
    printTerm(f"[LST2.0] ... Setting IP {subnet+str(address)}")
    # Define default gateway of nodes
    if bridge == nodes['brint']: node.setDefaultGateway(int_gateway, bridge)
    if bridge == nodes['brex']:  node.setDefaultGateway(ex_gateway , bridge)
    printTerm(f"[LST2.0] ... Setting Default Gateway")
    # Add routes to enable nodes within internal subnet communicate with server subnet
    if subnet != server_subnet: node.addRoute(server_subnet+'0', 24, bridge)
    if subnet != management_subnet: node.addRoute(management_subnet+'0', 24, bridge)
    if subnet != office_subnet: node.addRoute(office_subnet+'0', 24, bridge)
    if subnet != developer_subnet: node.addRoute(developer_subnet+'0', 24, bridge)
    if subnet != external_subnet: node.addRoute(external_subnet+'0', 24, bridge)
    printTerm(f"[LST2.0] ... Adding new routes")
    if setFiles:
        subprocess.run(f"docker cp serverconfig.ini {node.getNodeName()}:/home/debian/serverconfig.ini", shell=True)
        subprocess.run(f"docker cp backup.py {node.getNodeName()}:/home/debian/backup.py", shell=True)


def createBridge(name: str, ip: str, gatewayIp: str, netflowPort=9000) -> None: 
    printTerm(f"[LST2.0] Creating Switch {name}")
    nodes[name] = Switch(name, getcwd()+'/flows/'+name, '/home/pcap')
    nodes[name].run('mkdir /home/pcap > /dev/null 2>&1')
    nodes[name].instantiate()
    printTerm("[LST2.0] ... Instantiating container")
    nodes[name].setIp(ip, 24)
    printTerm(f"[LST2.0] ... Setting IP {ip}")
    nodes[name].connectToInternet(gatewayIp, 24)
    printTerm(f"[LST2.0] ... Connecting to Internet through Host")


def createController(name: str, bridgeName: str, controllerIp: str, controllerPort: int) -> None:
    printTerm(f"[LST2.0] Creating Controller {name}")
    nodes[name] = Controller(name)
    nodes[name].instantiate()
    printTerm("[LST2.0] ... Instantiating container")
    nodes[name].connect(nodes[bridgeName])
    printTerm(f"[LST2.0] ... Connecting to Node {bridgeName}")
    nodes[name].setIp(controllerIp, 24, nodes[bridgeName])
    printTerm(f"[LST2.0] ... Setting IP {controllerIp}")
    nodes[name].initController(controllerIp, controllerPort)
    printTerm(f"[LST2.0] ... Initiating Ryu Controller at {controllerIp}:{controllerPort}")
    nodes[bridgeName].setController(controllerIp, controllerPort)
    printTerm(f"[LST2.0] ... Conecting Switch {bridgeName} to controller")
    

def createServer(name: str, serverImage: str, subnet: str,  address: int) -> None:
    printTerm(f"[LST2.0] Creating Server {name}")
    nodes[name] = Host(name)
    nodes[name].instantiate(serverImage)
    printTerm("[LST2.0] ... Instantiating container")
    setNetworkConfig(nodes[name], nodes['brint'], subnet, address)


def createLinuxClient(name, bridge: Node, subnet: str, address: int) -> None:
    printTerm(f"[LST2.0] Creating Client {name}")
    nodes[name] = LinuxClient(name)
    nodes[name].instantiate(linuxclient)
    printTerm("[LST2.0] ... Instantiating container")
    setNetworkConfig(nodes[name], bridge, subnet, address)


def createPrinter(name: str, subnet: str) -> None:
    printTerm(f"[LST2.0] Creating Printer {name}")
    nodes[name] = Host(name)
    nodes[name].instantiate(printerserver)
    printTerm("[LST2.0] ... Instantiating container")
    setNetworkConfig(nodes[name], nodes['brint'], subnet, 1)


def unmakeChanges():
    printTerm(f"[LST2.0] Unmaking Experiment. Deleting Containers")
    [node.delete() for _,node in nodes.items()]


def signal_handler(sig, frame):
    print('You pressed Ctrl+C!')
    unmakeChanges()
    sys.exit(0)


def collectLogs():
    printTerm(f"[LST2.0] Collecting Client Logs")
    hosts = ['e1', 'e2', 'm1', 'm2', 'm3', 'm4', 'o1', 'o2', 'd1', 'd2', 'd3', 'd4', 'd5', 'd6', 'd7', 'd8', 'd9', 'd10', 'd11', 'd12', 'd13']
    ips = ['50.3', '50.4', '200.2', '200.3', '200.4', '200.5', '210.2', '210.3', '220.2', '220.3', '220.4', '220.5', '220.6', '220.7', '220.8', '220.9', '220.10', '220.11', '220.12', '220.13', '220.14']
    def getLog(ip, host):
        subprocess.run(f'docker cp {host}:/home/debian/log/192.168.{ip}.log logs/192.168.{ip}.log > /dev/null 2>&1', shell=True)
    [getLog(ip, host) for ip, host in zip(ips, hosts)]


def printTerm(string: str) -> None:
    subprocess.run(f"echo \"{string}\"", shell=True)


# Capture the ctrl+c
signal.signal(signal.SIGINT, signal_handler)

try:
    printTerm("LST 2.0  Copyright (C) 2022\n\n\tThis program comes with ABSOLUTELY NO WARRANTY;\n\tThis is free software, and you are welcome to redistribute it\n\tunder certain conditions;\n\n")
    printTerm("[LST2.0] Iniciando o Experimento")    
    # Create folder to store the 
    subprocess.run("mkdir flows 2>/dev/null", shell=True)

    # Create Bridges and connect them
    createBridge('brint', brint_ip, int_gateway)
    createBridge('brex', brex_ip, ex_gateway)
    nodes['brex'].connect(nodes['brint'])

    subprocess.run(f"ip route add 192.168.200.0/24 dev veth-host-brint", shell=True)
    subprocess.run(f"ip route add 192.168.210.0/24 dev veth-host-brint", shell=True)
    subprocess.run(f"ip route add 192.168.220.0/24 dev veth-host-brint", shell=True)

    # Create seafile server
    nodes['seafile'] = Seafile('seafile')
    nodes['seafile'].instantiate()
    setNetworkConfig(nodes['seafile'], nodes['brint'], external_subnet, 1, setFiles=False)
    nodes['seafile'].updateServerConfig()

    # Create controllers
    createController('c1', 'brint', c1_ip, c1port)
    createController('c2', 'brex', c1_ip, c1port)

    # Create server subnet
    createServer('mail',   mailserver,   server_subnet, 1)
    createServer('file',   fileserver,   server_subnet, 2)
    createServer('web',    webserver,    server_subnet, 3)
    createServer('backup', backupserver, server_subnet, 4)

    # Set Management Subnet
    createPrinter('mprinter', management_subnet)
    createLinuxClient('m1', nodes['brint'], management_subnet, 2)
    createLinuxClient('m2', nodes['brint'], management_subnet, 3)
    createLinuxClient('m3', nodes['brint'], management_subnet, 4)
    createLinuxClient('m4', nodes['brint'], management_subnet, 5)
        
    # Set Office Subnet
    #createPrinter('oprinter', office_subnet)
    #createLinuxClient('o1', nodes['brint'], office_subnet, 2)
    #createLinuxClient('o2', nodes['brint'], office_subnet, 3)

    # Set Developer Subnet
    #createPrinter('dprinter', developer_subnet)
    #createLinuxClient('d1', nodes['brint'], developer_subnet, 2)
    #createLinuxClient('d2', nodes['brint'], developer_subnet, 3)
    #createLinuxClient('d3', nodes['brint'], developer_subnet, 4)
    #createLinuxClient('d4', nodes['brint'], developer_subnet, 5)
    #createLinuxClient('d5', nodes['brint'], developer_subnet, 6)
    #createLinuxClient('d6', nodes['brint'], developer_subnet, 7)
    #createLinuxClient('d7', nodes['brint'], developer_subnet, 8)
    #createLinuxClient('d8', nodes['brint'], developer_subnet, 9)
    #createLinuxClient('d9', nodes['brint'], developer_subnet, 10)
    #createLinuxClient('d10', nodes['brint'], developer_subnet, 11)
    #createLinuxClient('d11', nodes['brint'], developer_subnet, 12)
    #createLinuxClient('d12', nodes['brint'], developer_subnet, 13)
    #createLinuxClient('d13', nodes['brint'], developer_subnet, 14)

    # Set External Subnet
    #createServer('eweb', webserver, external_subnet, 2)
    #createLinuxClient('e1', nodes['brex'], external_subnet, 3)
    #createLinuxClient('e2', nodes['brex'], external_subnet, 4)

    # Set Configuration Files
    [setLinuxClientFileConfig(nodes[f'm{i}'], management_subnet, 'management') for i in range(1, 5)]
    #[setLinuxClientFileConfig(nodes[f'o{i}'], office_subnet, 'office') for i in range(1,3)]
    #[setLinuxClientFileConfig(nodes[f'd{i}'], developer_subnet, 'administrator') for i in range(1,3)]
    #[setLinuxClientFileConfig(nodes[f'd{i}'], developer_subnet, 'developer') for i in range(3,12)]
    #[setLinuxClientFileConfig(nodes[f'd{i}'], developer_subnet, 'attacker') for i in range(12,14)]
    #[setLinuxClientFileConfig(nodes[f'e{i}'], developer_subnet, 'external_attacker') for i in range(1,3)]

    nodes['brint'].collectFlows(path='home/pcap', sniffAll=True)
    nodes['brex'].collectFlows(path='home/pcap', sniffAll=True)
except Exception as ex:
    collectLogs()
    unmakeChanges()
    raise Exception(str(ex))

print('[LST2.0] Press CTRL+C to finish experiment')
signal.pause()



from time import time, sleep
from .deploy_mininet import DeployMininet


dmn = DeployMininet()
nodes = 100
deployTime = {}
undeployTime = {}


for i in range(100):
	start = time()
	dmn.deploy(i)
	deployTime[i] = time() - start
	sleep(3)
	start = time()
	dmn.undeploy()
	undeployTime[i] = time() - start
	sleep(3)


print(deployTime)
print(undeployTime)



from experiment.constants import *
from profissa_lft.ue import UE
from subprocess import run

class EmuPhyWireless:
    def __init__(self):
        self.ue = UE('ue1')

    def setup(self):
        self.ue.instantiate(dockerImage=SRSRAN_PERFSONAR_UHD_IMAGE, runCommand=USR_SBIN_INIT_COMMAND)
        self.ue.connectToInternet(EMU_PHY_WIRELESS_UE_IP_ADDR, 29, "uehost", "hostue")
        self.ue.setIp('10.0.0.2', 29, "uehost")
        self.ue.setDefaultGateway(EMU_PHY_WIRELESS_UE_IP_ADDR, "uehost")
        self.ue.setMtuSize("uehost", 9000)
        run("ifconfig hostue mtu 9000", shell=True)

        self.ue.setDeviceName("uhd")
        self.ue.setTxGain(str(25))
        self.ue.setRxGain(str(25))
        self.ue.setDeviceArgs(f"type=x300,addr={EMU_PHY_WIRELESS_USRP_IP_ADDR}")

        self.ue.setHost(EMU_PHY_WIRELESS_UE_IP_ADDR)
        self.ue.start()
        

    def tearDown(self):
        self.ue.delete()


# Copyright (C) 2022 Alexandre Mitsuru Kaihara
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


# Brief: This exception is for in case the terminal input is incorrect
class InvalidCommandLineInput(Exception):
    pass

# Brief: This expection is related a invalid parameter
class MissingObjectParameter(Exception):
    pass

# Brief: This exception is related to a failure during docker container instantiation 
class NodeInstantiationFailed(Exception):
    pass

# Brief: This exception is related to the creation of a 
class InvalidNodeName(Exception):
    pass


# Copyright (C) 2023 Alexandre Mitsuru Kaihara
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .perfsonar import Perfsonar
from configparser import ConfigParser
from .constants import *

class UE(Perfsonar):
    def __init__(self, name: str, ueConfigPath='/etc/srsran/ue.conf', buildDir='/srsRAN/build'):
        super().__init__(name)
        self.configPath = ueConfigPath
        self.config = None
        self.buildDir = buildDir

    def instantiate(self, dockerImage='alexandremitsurukaihara/lft:srsran', dockerCommand = '', dns='8.8.8.8', runCommand='', cpus='', memory='') -> None:
        super().instantiate(dockerImage, dockerCommand, dns, runCommand=runCommand, cpus=cpus, memory=memory)
        self.config = self.readConfigFile(self.configPath)

    def start(self, deviceArgs='') -> None:
        # , transmitterIp="*", transmitterPort=2001, receiverIp="localhost", receiverPort=2000
        #args = "--rf.device_name=zmq --rf.device_args=\"tx_port=tcp://{transmitterIp}:{transmitterPort},rx_port=tcp://{receiverIp}:{receiverPort},id=ue,base_srate=23.04e6\""
        super().run(f"{self.buildDir}/srsue/src/srsue {deviceArgs} > {self.buildDir}/ue.out &")
    
    def stop(self) -> None:
        self.run(f"pkill -f -9 srsue")

    def setConfigPath(self, path: str) -> None:
        self.configPath = path

    def getConfigPath(self) -> str:
        return self.configPath

    def setConfigurationFile(self, filePath: str, destinationPath='') -> None:
        if destinationPath == '':
            destinationPath = self.configPath
        super().copyLocalToContainer(filePath, destinationPath)

    def setDeviceArgs(self, deviceArgs: str) -> None:
        self.config[RF_SECTION][DEVICE_ARGS_ATTR] = deviceArgs
        self.saveConfig(self.config, self.configPath)

    def setDeviceName(self, deviceName: str) -> None:
        self.config[RF_SECTION][DEVICE_NAME_ATTR] = deviceName
        self.saveConfig(self.config, self.configPath)

    def setTxGain(self, txGain: int) -> None:
        self.config[RF_SECTION][TX_GAIN_ATTR] = txGain
        self.saveConfig(self.config, self.configPath)

    def setRxGain(self, rxGain: int) -> None:
        self.config[RF_SECTION][RX_GAIN_ATTR] = rxGain
        self.saveConfig(self.config, self.configPath)

    def setAuthenticationAlgorithm(self, algorithmName: str) -> None:
        self.config[USIM_SECTION][ALGORITHM_ATTR] = algorithmName
        self.saveConfig(self.config, self.configPath)

    def setUEID(self, id: str) -> None:
        self.config[USIM_SECTION][IMSI_ATTR] = id
        self.saveConfig(self.config, self.configPath)
        
    def setCorrectSyncError(self, enable: bool) -> None:
        self.config[PHY_SECTION][CORRECT_SYNC_ERROR] = "true" if enable else "false"
        self.saveConfig(self.config, self.configPath)
        



repository = 'alexandremitsurukaihara/lst2.0'
seafileserver = repository + ':seafile'
mailserver = repository + ':mail'
webserver = repository + ':web'
fileserver = repository + ':file'
backupserver = repository + ':backup'
printerserver = repository + ':printer'
linuxclient = repository + ':linuxclient'
brint_ip = '192.168.100.100'
int_gateway = '192.168.100.101'
c1_ip = '192.168.100.102'
server_subnet = '192.168.100.'
management_subnet = '192.168.200.'
office_subnet = '192.168.210.'
developer_subnet = '192.168.220.'
external_subnet = '192.168.50.'
brex_ip = external_subnet+'100'
ex_gateway= '192.168.50.101'
c2_ip = '192.168.50.102'
c1port = 9001
c2port = 9001
nodes = {}


from .node import Node
from json import load, dump


class Perfsonar(Node):
    def instantiate(self, dockerImage='alexandremitsurukaihara/lft:srsran', dockerCommand = '', dns='8.8.8.8', runCommand='', cpus='', memory='') -> None:
        super().instantiate(dockerImage=dockerImage, dockerCommand=dockerCommand, dns=dns, runCommand=runCommand, cpus=cpus, memory=memory)

    def readLimitFile(self, limitPath="/etc/pscheduler/limits.conf"):
        randomTmpName = self.getHashFromString(limitPath) 
        self.copyContainerToLocal(limitPath, f"/tmp/lft/{randomTmpName}")

        with open(f"/tmp/lft/{randomTmpName}") as f:
            self.limitData = load(f)

    def saveLimitFile(self, limitPath="/etc/pscheduler/limits.conf"):
        randomTmpName = self.getHashFromString(limitPath)
        with open(f"/tmp/lft/{randomTmpName}", "w") as f:
            dump(self.limitData, f)
        self.copyLocalToContainer(f"/tmp/lft/{randomTmpName}", limitPath)

    def addRouteException(self, ip: str, netmask: int) -> None:
        self.limitData['identifiers'][2]['data']['exclude'].append(f"{ip}/{netmask}")




# Copyright (C) 2023 Alexandre Mitsuru Kaihara
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .perfsonar import Perfsonar
from .constants import *

class EnB(Perfsonar):
    def __init__(self, name: str, eNBConfigPath = '/etc/srsran/enb.conf'):
        super().__init__(name)
        self.defaultEnBConfigPath = eNBConfigPath
        self.buildDir = '/srsRAN/build'
        self.config = None
        self.defaultMultiUEPath = self.buildDir + '/multiUE.py'
        self.defaultSingleUEPath = self.buildDir + '/singleUE.py'

    def instantiate(self, dockerImage='alexandremitsurukaihara/lft:srsran', dockerCommand = '', dns='8.8.8.8', runCommand='') -> None:
        super().instantiate(dockerImage=dockerImage, dockerCommand=dockerCommand, dns=dns, runCommand=runCommand)
        self.config = self.readConfigFile(self.defaultEnBConfigPath)

    def start(self, transmitterIp="*", transmitterPort=2000, receiverIp="localhost", receiverPort=2001) -> None:
        super().run(f"{self.buildDir}/srsenb/src/srsenb --rf.device_name=zmq --rf.device_args=\'fail_on_disconnect=true,tx_port=tcp://{transmitterIp}:{transmitterPort},rx_port=tcp://{receiverIp}:{receiverPort},id=enb,base_srate=34.56e6\' > enb.log")

    def stop(self) -> None:
        super().run("pkill -f -9 srsenb")

    def setDefaultEnBConfigPath(self, path: str) -> None:
        self.defaultEnBConfigPath = path

    def getdefaultEnBConfigPath(self) -> None:
        return self.defaultEnBConfigPath

    def setConfigurationFile(self, filePath: str, destinationPath='') -> None:
        if destinationPath == '':
            destinationPath = self.defaultEnBConfigPath
        super().copyLocalToContainer(filePath, destinationPath)

    # This option make sense only for emulated radio
    def starGnuRadioMultiUE(self, multiUEPath='') -> None:
        if multiUEPath == '':
            multiUEPath = self.defaultMultiUEPath
        super().run(f"python3 {multiUEPath}")

    def starGnuRadioSingleUE(self, singleUEPath='') -> None:
        if singleUEPath == '':
            singleUEPath = self.defaultSingleUEPath
        super().run(f"python3 {singleUEPath}")

    def stopGnuRadioMultiUE(self) -> None:
        super().run(f"pkill -f -9 multiUE")

    def setMultiUEEnBAddr(self, txIP: str, txPort: int, rxIP: str, rxPort: int, multiUEPath='') -> None:
        if multiUEPath == '':
            multiUEPath = self.defaultMultiUEPath
        self.run(f"sed -i \"/zeromq_req_source_2/s@\'tcp://.*\'@\'tcp://{txIP}:{txPort}\'@\" {multiUEPath}")
        self.run(f"sed -i \"/zeromq_rep_sink_0/s@'tcp://.*'@'tcp://{rxIP}:{rxPort}'@\" {multiUEPath}")

    def setMultiUEUE1Addr(self, UEIP: str, txPort: int, eNBIP: str, rxPort: int, multiUEPath='') -> None:
        if multiUEPath == '':
            multiUEPath = self.defaultMultiUEPath
        self.run(f"sed -i \"/zeromq_req_source_1/s@'tcp://.*'@'tcp://{UEIP}:{txPort}'@\" {multiUEPath}")
        self.run(f"sed -i \"/zeromq_rep_sink_2/s@'tcp://.*'@'tcp://{eNBIP}:{rxPort}'@\" {multiUEPath}")

    def setMultiUEUE2Addr(self, UEIP: str, txPort: int, eNBIP: str, rxPort: int, multiUEPath='') -> None:
        if multiUEPath == '':
            multiUEPath = self.defaultMultiUEPath
        self.run(f"sed -i \"/zeromq_req_source_0/s@'tcp://.*'@'tcp://{UEIP}:{txPort}'@\" {multiUEPath}")
        self.run(f"sed -i \"/zeromq_rep_sink_1/s@'tcp://.*'@'tcp://{eNBIP}:{rxPort}'@\" {multiUEPath}")

    def setSingleUEEnBAddr(self, txIP: str, txPort: int, rxIP: str, rxPort: int) -> None:
        self.run(f"sed -i \"/zeromq_req_source_enb/s@\'tcp://.*\'@\'tcp://{txIP}:{txPort}\'@\" {self.defaultSingleUEPath}")
        self.run(f"sed -i \"/zeromq_rep_sink_enb/s@'tcp://.*'@'tcp://{rxIP}:{rxPort}'@\" {self.defaultSingleUEPath}")

    def setSingleUEUEAddr(self, UEIP: str, txPort: int, eNBIP: str, rxPort: int) -> None:
        self.run(f"sed -i \"/zeromq_req_source_ue/s@'tcp://.*'@'tcp://{UEIP}:{txPort}'@\" {self.defaultSingleUEPath}")
        self.run(f"sed -i \"/zeromq_rep_sink_ue/s@'tcp://.*'@'tcp://{eNBIP}:{rxPort}'@\" {self.defaultSingleUEPath}")

    def setDeviceArgs(self, deviceArgs: str) -> None:
        self.config[RF_SECTION][DEVICE_ARGS_ATTR] = deviceArgs
        self.saveConfig(self.config, self.defaultEnBConfigPath)

    def setDeviceName(self, deviceName: str) -> None:
        self.config[RF_SECTION][DEVICE_NAME_ATTR] = deviceName
        self.saveConfig(self.config, self.defaultEnBConfigPath)

    def setEPCAddress(self, ip: str) -> None:
        self.config[ENB_SECTION][MME_ADDR] = ip
        self.saveConfig(self.config, self.defaultEnBConfigPath)

    def setEnBAddress(self, ip: str) -> None:
        self.config[ENB_SECTION][GTP_BIND_ADDR] = ip
        self.config[ENB_SECTION][S1C_BIND_ADDR] = ip
        self.saveConfig(self.config, self.defaultEnBConfigPath)



# Copyright (C) 2024 Alexandre Mitsuru Kaihara
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# This file is for defining a package in Python
from setuptools import setup, find_packages

setup(
    name='profissa_lft',
    version='1.0.5',
    packages=find_packages(),
    install_requires=['pandas'],
    author='Alexandre Mitsuru Kaihara',
    author_email='alexandreamk1@gmail.com',
    description='LFT is a framework designed to facilitate the creation of lightweight network topologies with ease. Using Docker containers, it is possible to add any container to the network to provide network services or even emulate network devices, such as switches, controllers (in Software Defined Networking). This project has integration with OpenvSwitch to emulate the network forwarding devices and srsRAN 4G to emulate wireless links for Fog and Edge application scenarios.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/alexandrekaihara/lft    ',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.9',
)


from .experiment import *
from .emu_emu_wireless import EmuEmuWireless
from .emu_phy_wireless import EmuPhyWireless


emuEmu = EmuEmuWireless()
emuEmu.setup()
runThroughput(EMU_EMU_WIRELESS_PREFIX, EMU_EMU_WIRELESS_EPC_IP_ADDR, EMU_EMU_WIRELESS_UE_IP_ADDR)
runRTT(EMU_EMU_WIRELESS_PREFIX, EMU_EMU_WIRELESS_EPC_IP_ADDR, EMU_EMU_WIRELESS_UE_IP_ADDR)
runLatency(EMU_EMU_WIRELESS_PREFIX, EMU_EMU_WIRELESS_EPC_IP_ADDR, EMU_EMU_WIRELESS_UE_IP_ADDR)
emuEmu.tearDown()

emuPhy = EmuPhyWireless()
emuPhy.setup()
runThroughput(EMU_PHY_WIRELESS_PREFIX, EMU_PHY_WIRELESS_EPC_IP_ADDR, EMU_PHY_WIRELESS_UE_IP_ADDR)
runRTT(EMU_PHY_WIRELESS_PREFIX, EMU_PHY_WIRELESS_EPC_IP_ADDR, EMU_PHY_WIRELESS_UE_IP_ADDR)
runLatency(EMU_PHY_WIRELESS_PREFIX, EMU_PHY_WIRELESS_EPC_IP_ADDR, EMU_PHY_WIRELESS_UE_IP_ADDR)
emuPhy.tearDown()

runThroughput(PHY_PHY_WIRELESS_PREFIX, PHY_PHY_WIRELESS_EPC_IP_ADDR, PHY_PHY_WIRELESS_UE_IP_ADDR)
runRTT(PHY_PHY_WIRELESS_PREFIX, PHY_PHY_WIRELESS_EPC_IP_ADDR, PHY_PHY_WIRELESS_UE_IP_ADDR)
runLatency(PHY_PHY_WIRELESS_PREFIX, PHY_PHY_WIRELESS_EPC_IP_ADDR, PHY_PHY_WIRELESS_UE_IP_ADDR)

    


from profissa_lft.ue import UE
from profissa_lft.epc import EPC
from profissa_lft.enb import EnB
import time
import subprocess
from time import sleep
from experiment.constants import *

class EmuEmuWireless:
    def __init__(self):
        self.epc = EPC('epc')
        self.enb = EnB("enb")
        self.ue = UE('ue')

    def setup(self):
        self.epc.instantiate(dockerImage=SRSRAN_PERFSONAR_UHD_IMAGE, runCommand=USR_SBIN_INIT_COMMAND)
        self.enb.instantiate(dockerImage=SRSRAN_PERFSONAR_UHD_IMAGE)
        self.ue.instantiate(dockerImage=SRSRAN_PERFSONAR_UHD_IMAGE, runCommand=USR_SBIN_INIT_COMMAND)

        self.enb.connect(self.epc, "enbepc", "epcenb")
        self.ue.connect(self.enb, "ueenb", "enbue")

        self.epc.connectToInternet('9.0.0.2', 24, "epchost", "hostepc")
        self.ue.connectToInternet('12.0.0.2', 24, "uehost", "hostue")

        self.epc.setIp('9.0.0.1', 24, "epchost")
        self.epc.setIp('10.0.0.1', 24, "epcenb")
        self.enb.setIp('10.0.0.2', 24, "enbepc")
        self.enb.setIp('11.0.0.1', 30, "enbue")
        self.ue.setIp('12.0.0.1', 24, "uehost")
        self.ue.setIp('11.0.0.2', 29, "ueenb")

        self.epc.setDefaultGateway('9.0.0.2', "epchost")
        self.enb.setDefaultGateway('9.0.0.2', "enbepc")
        self.ue.setDefaultGateway('12.0.0.2', "ueenb")

        self.enb.acceptPacketsFromInterface("enbue")

        # Define EPC config
        self.epc.setEPCAddress("10.0.0.1")
        self.epc.addNewUE(self.ue.getNodeName(), "001010123456780", EMU_EMU_WIRELESS_UE_IP_ADDR)

        # Define ENB Config
        self.enb.setEPCAddress("10.0.0.1")
        self.enb.setEnBAddress("10.0.0.2")
        self.enb.setSingleUEEnBAddr("11.0.0.1", 2101, '11.0.0.1', 2100)
        self.enb.setSingleUEUEAddr("11.0.0.2", 2001, "11.0.0.1", 2000)

        # Define UE Config
        self.ue.setUEID("001010123456780")
        self.ue.setCorrectSyncError(True)

        self.epc.start()
        self.enb.starGnuRadioSingleUE()
        self.enb.start("11.0.0.1", 2101, "11.0.0.1", 2100)
        time.sleep(5)

        self.ue.start("--rf.device_name=zmq --rf.device_args=\"tx_port=tcp://11.0.0.2:2001,rx_port=tcp://11.0.0.1:2000,id=ue,base_srate=34.56e6\"")

        self.ue.setHost(EMU_EMU_WIRELESS_UE_IP_ADDR)
        self.epc.setHost(EMU_EMU_WIRELESS_EPC_IP_ADDR)

        self.epc.enableForwarding("epchost", "srs_spgw_sgi")
        self.ue.enableForwarding("uehost", "tun_srsue")

        subprocess.run(f"ip route add {EMU_EMU_WIRELESS_UE_IP_ADDR}/32 via 12.0.0.1 dev hostue", shell=True)
        subprocess.run(f"ip route add {EMU_EMU_WIRELESS_EPC_IP_ADDR}/32 via 9.0.0.1 dev hostepc", shell=True)

        self.__startPerfsonarServices()

    def __startPerfsonarServices(self):
         self.epc.run("service pscheduler-runner start && service pscheduler-ticker start && service pscheduler-scheduler start && service pscheduler-archiver start")
         sleep(5)
         self.ue.run("service pscheduler-runner start && service pscheduler-ticker start && service pscheduler-scheduler start && service pscheduler-archiver start")

    def tearDown(self):
        self.epc.delete()
        self.enb.delete()
        self.ue.delete()



"""
Node Library for Mininet

This contains additional Node types which you may find to be useful.
"""

from mininet.node import Node, Switch
from mininet.log import info, warn
from mininet.moduledeps import pathCheck
from mininet.util import quietRun


class LinuxBridge( Switch ):
    "Linux Bridge (with optional spanning tree)"

    nextPrio = 100  # next bridge priority for spanning tree

    def __init__( self, name, stp=False, prio=None, **kwargs ):
        """stp: use spanning tree protocol? (default False)
           prio: optional explicit bridge priority for STP"""
        self.stp = stp
        if prio:
            self.prio = prio
        else:
            self.prio = LinuxBridge.nextPrio
            LinuxBridge.nextPrio += 1
        Switch.__init__( self, name, **kwargs )

    def connected( self ):
        "Are we forwarding yet?"
        if self.stp:
            return 'forwarding' in self.cmd( 'brctl showstp', self )
        else:
            return True

    def start( self, _controllers ):
        "Start Linux bridge"
        self.cmd( 'ifconfig', self, 'down' )
        self.cmd( 'brctl delbr', self )
        self.cmd( 'brctl addbr', self )
        if self.stp:
            self.cmd( 'brctl setbridgeprio', self.prio )
            self.cmd( 'brctl stp', self, 'on' )
        for i in self.intfList():
            if self.name in i.name:
                self.cmd( 'brctl addif', self, i )
        self.cmd( 'ifconfig', self, 'up' )

    def stop( self, deleteIntfs=True ):
        """Stop Linux bridge
           deleteIntfs: delete interfaces? (True)"""
        self.cmd( 'ifconfig', self, 'down' )
        self.cmd( 'brctl delbr', self )
        super( LinuxBridge, self ).stop( deleteIntfs )

    def dpctl( self, *args ):
        "Run brctl command"
        return self.cmd( 'brctl', *args )

    @classmethod
    def setup( cls ):
        "Check dependencies and warn about firewalling"
        pathCheck( 'brctl', moduleName='bridge-utils' )
        # Disable Linux bridge firewalling so that traffic can flow!
        for table in 'arp', 'ip', 'ip6':
            cmd = 'sysctl net.bridge.bridge-nf-call-%stables' % table
            out = quietRun( cmd ).strip()
            if out.endswith( '1' ):
                warn( 'Warning: Linux bridge may not work with', out, '\n' )


class NAT( Node ):
    "NAT: Provides connectivity to external network"

    def __init__( self, name, subnet='10.0/8',
                  localIntf=None, flush=False, **params):
        """Start NAT/forwarding between Mininet and external network
           subnet: Mininet subnet (default 10.0/8)
           flush: flush iptables before installing NAT rules"""
        super( NAT, self ).__init__( name, **params )

        self.subnet = subnet
        self.localIntf = localIntf
        self.flush = flush
        self.forwardState = self.cmd( 'sysctl -n net.ipv4.ip_forward' ).strip()

    def config( self, **params ):
        """Configure the NAT and iptables"""
        super( NAT, self).config( **params )

        if not self.localIntf:
            self.localIntf = self.defaultIntf()

        if self.flush:
            self.cmd( 'sysctl net.ipv4.ip_forward=0' )
            self.cmd( 'iptables -F' )
            self.cmd( 'iptables -t nat -F' )
            # Create default entries for unmatched traffic
            self.cmd( 'iptables -P INPUT ACCEPT' )
            self.cmd( 'iptables -P OUTPUT ACCEPT' )
            self.cmd( 'iptables -P FORWARD DROP' )

        # Install NAT rules
        self.cmd( 'iptables -I FORWARD',
                  '-i', self.localIntf, '-d', self.subnet, '-j DROP' )
        self.cmd( 'iptables -A FORWARD',
                  '-i', self.localIntf, '-s', self.subnet, '-j ACCEPT' )
        self.cmd( 'iptables -A FORWARD',
                  '-o', self.localIntf, '-d', self.subnet, '-j ACCEPT' )
        self.cmd( 'iptables -t nat -A POSTROUTING',
                  '-s', self.subnet, "'!'", '-d', self.subnet,
                  '-j MASQUERADE' )

        # Instruct the kernel to perform forwarding
        self.cmd( 'sysctl net.ipv4.ip_forward=1' )

        # Prevent network-manager from messing with our interface
        # by specifying manual configuration in /etc/network/interfaces
        intf = self.localIntf
        cfile = '/etc/network/interfaces'
        line = '\niface %s inet manual\n' % intf
        config = open( cfile ).read()
        if ( line ) not in config:
            info( '*** Adding "' + line.strip() + '" to ' + cfile + '\n' )
            with open( cfile, 'a' ) as f:
                f.write( line )
        # Probably need to restart network-manager to be safe -
        # hopefully this won't disconnect you
        self.cmd( 'service network-manager restart' )

    def terminate( self ):
        "Stop NAT/forwarding between Mininet and external network"
        # Remote NAT rules
        self.cmd( 'iptables -D FORWARD',
                   '-i', self.localIntf, '-d', self.subnet, '-j DROP' )
        self.cmd( 'iptables -D FORWARD',
                  '-i', self.localIntf, '-s', self.subnet, '-j ACCEPT' )
        self.cmd( 'iptables -D FORWARD',
                  '-o', self.localIntf, '-d', self.subnet, '-j ACCEPT' )
        self.cmd( 'iptables -t nat -D POSTROUTING',
                  '-s', self.subnet, '\'!\'', '-d', self.subnet,
                  '-j MASQUERADE' )
        # Put the forwarding state back to what it was
        self.cmd( 'sysctl net.ipv4.ip_forward=%s' % self.forwardState )
        super( NAT, self ).terminate()



"""
link.py: interface and link abstractions for mininet
It seems useful to bundle functionality for interfaces into a single
class.
Also it seems useful to enable the possibility of multiple flavors of
links, including:
- simple veth pairs
- tunneled links
- patchable links (which can be disconnected and reconnected via a patchbay)
- link simulators (e.g. wireless)
Basic division of labor:
  Nodes: know how to execute commands
  Intfs: know how to configure themselves
  Links: know how to connect nodes together
Intf: basic interface object that can configure itself
TCIntf: interface with bandwidth limiting and delay via tc
Link: basic link class for creating veth pairs
"""

from mininet.log import info, error, debug
from containernet.util import makeIntfPair
import re


class Intf( object ):

    "Basic interface object that can configure itself."

    def __init__( self, name, node=None, port=None, link=None,
                  mac=None, **params ):
        """name: interface name (e.g. h1-eth0)
           node: owning node (where this intf most likely lives)
           link: parent link if we're part of a link
           other arguments are passed to config()"""
        self.node = node
        self.name = name
        self.link = link
        self.mac = mac
        self.ip, self.prefixLen = None, None

        # if interface is lo, we know the ip is 127.0.0.1.
        # This saves an ifconfig command per node
        if self.name == 'lo':
            self.ip = '127.0.0.1'
            self.prefixLen = 8
        # Add to node (and move ourselves if necessary )
        if node:
            moveIntfFn = params.pop( 'moveIntfFn', None )
            if moveIntfFn:
                node.addIntf( self, port=port, moveIntfFn=moveIntfFn )
            else:
                node.addIntf( self, port=port )
        # Save params for future reference
        self.params = params
        self.config( **params )

    def cmd( self, *args, **kwargs ):
        "Run a command in our owning node"
        return self.node.cmd( *args, **kwargs )

    def ifconfig( self, *args ):
        "Configure ourselves using ifconfig"
        return self.cmd( 'ifconfig', self.name, *args )

    def setIP( self, ipstr, prefixLen=None, overwrite=True ):
        """Set our IP address"""
        # This is a sign that we should perhaps rethink our prefix
        # mechanism and/or the way we specify IP addresses

        # remove all old adresses (to remove side effects)
        # this is needed after the swtich from "ifconfig" to "ip"
        if overwrite:
            self.cmd("ip", "address", "flush", "dev", self.name)
        # add the new address
        if '/' in ipstr:
            self.ip, self.prefixLen = ipstr.split( '/' )
            return self.cmd('ip', 'address', 'add', ipstr, 'dev', self.name)
        else:
            if prefixLen is None:
                raise Exception( 'No prefix length set for IP address %s'
                                 % ( ipstr, ) )
            self.ip, self.prefixLen = ipstr, prefixLen
            return self.cmd('ip', 'address', 'add', '%s/%s' % (ipstr, prefixLen), 'dev', self.name)

    def setMAC( self, macstr ):
        """Set the MAC address for an interface.
           macstr: MAC address as string"""
        self.mac = macstr
        return ( self.ifconfig( 'down' ) +
                 self.ifconfig( 'hw', 'ether', macstr ) +
                 self.ifconfig( 'up' ) )

    _ipMatchRegex = re.compile( r'\d+\.\d+\.\d+\.\d+' )
    _macMatchRegex = re.compile( r'..:..:..:..:..:..' )

    def updateIP( self ):
        "Return updated IP address based on ifconfig"
        # use pexec instead of node.cmd so that we dont read
        # backgrounded output from the cli.
        ifconfig, _err, _exitCode = self.node.pexec(
            'ifconfig %s' % self.name )
        ips = self._ipMatchRegex.findall( ifconfig )
        self.ip = ips[ 0 ] if ips else None
        return self.ip

    def updateMAC( self ):
        "Return updated MAC address based on ifconfig"
        ifconfig = self.ifconfig()
        macs = self._macMatchRegex.findall( ifconfig )
        self.mac = macs[ 0 ] if macs else None
        return self.mac

    # Instead of updating ip and mac separately,
    # use one ifconfig call to do it simultaneously.
    # This saves an ifconfig command, which improves performance.

    def updateAddr( self ):
        "Return IP address and MAC address based on ifconfig."
        ifconfig = self.ifconfig()
        ips = self._ipMatchRegex.findall( ifconfig )
        macs = self._macMatchRegex.findall( ifconfig )
        self.ip = ips[ 0 ] if ips else None
        self.mac = macs[ 0 ] if macs else None
        return self.ip, self.mac

    def IP( self ):
        "Return IP address"
        return self.ip

    def MAC( self ):
        "Return MAC address"
        return self.mac

    def isUp( self, setUp=False ):
        "Return whether interface is up"
        if setUp:
            cmdOutput = self.ifconfig( 'up' )
            # no output / command output indicates success
            if (len(cmdOutput) > 0
                    and "ifconfig" not in cmdOutput):
                error( "Error setting %s up: %s " % ( self.name, cmdOutput ) )
                return False
            else:
                return True
        else:
            return "UP" in self.ifconfig()

    def rename( self, newname ):
        "Rename interface"
        if self.node and self.name in self.node.nameToIntf:
            # rename intf in node's nameToIntf
            self.node.nameToIntf[newname] = self.node.nameToIntf.pop(self.name)
        self.ifconfig( 'down' )
        result = self.cmd( 'ip link set', self.name, 'name', newname )
        self.name = newname
        self.ifconfig( 'up' )
        return result

    # The reason why we configure things in this way is so
    # That the parameters can be listed and documented in
    # the config method.
    # Dealing with subclasses and superclasses is slightly
    # annoying, but at least the information is there!

    def setParam( self, results, method, **param ):
        """Internal method: configure a *single* parameter
           results: dict of results to update
           method: config method name
           param: arg=value (ignore if value=None)
           value may also be list or dict"""
        name, value = list( param.items() )[ 0 ]
        f = getattr( self, method, None )
        if not f or value is None:
            return
        if isinstance( value, list ):
            result = f( *value )
        elif isinstance( value, dict ):
            result = f( **value )
        else:
            result = f( value )
        results[ name ] = result
        return result

    def config( self, mac=None, ip=None, ifconfig=None,
                up=True, **_params ):
        """Configure Node according to (optional) parameters:
           mac: MAC address
           ip: IP address
           ifconfig: arbitrary interface configuration
           Subclasses should override this method and call
           the parent class's config(**params)"""
        # If we were overriding this method, we would call
        # the superclass config method here as follows:
        # r = Parent.config( **params )
        r = {}
        self.setParam( r, 'setMAC', mac=mac )
        self.setParam( r, 'setIP', ip=ip )
        self.setParam( r, 'isUp', up=up )
        self.setParam( r, 'ifconfig', ifconfig=ifconfig )
        return r

    def delete( self ):
        "Delete interface"
        self.cmd( 'ip link del ' + self.name )
        # We used to do this, but it slows us down:
        # if self.node.inNamespace:
        # Link may have been dumped into root NS
        # quietRun( 'ip link del ' + self.name )
        self.node.delIntf( self )
        self.link = None
        from containernet.node import OVSSwitch
        # call detach if we have a OVSSwitch (just to be sure)
        if isinstance( self.node, OVSSwitch ):
            self.node.detach(self)

    def status( self ):
        "Return intf status as a string"
        links, _err, _result = self.node.pexec( 'ip link show' )
        if self.name in links:
            return "OK"
        else:
            return "MISSING"

    def __repr__( self ):
        return '<%s %s>' % ( self.__class__.__name__, self.name )

    def __str__( self ):
        return self.name


class TCIntf( Intf ):
    """Interface customized by tc (traffic control) utility
       Allows specification of bandwidth limits (various methods)
       as well as delay, loss and max queue length"""

    # The parameters we use seem to work reasonably up to 1 Gb/sec
    # For higher data rates, we will probably need to change them.
    bwParamMax = 1000

    def bwCmds( self, bw=None, speedup=0, use_hfsc=False, use_tbf=False,
                latency_ms=None, enable_ecn=False, enable_red=False ):
        "Return tc commands to set bandwidth"

        cmds, parent = [], ' root '

        if bw and ( bw < 0 or bw > self.bwParamMax ):
            error( 'Bandwidth limit', bw, 'is outside supported range 0..%d'
                   % self.bwParamMax, '- ignoring\n' )
        elif bw is not None:
            # BL: this seems a bit brittle...
            if ( speedup > 0 and
                 self.node.name[0:1] == 's' ):
                bw = speedup
            # This may not be correct - we should look more closely
            # at the semantics of burst (and cburst) to make sure we
            # are specifying the correct sizes. For now I have used
            # the same settings we had in the mininet-hifi code.
            if use_hfsc:
                cmds += [ '%s qdisc add dev %s root handle 5:0 hfsc default 1',
                          '%s class add dev %s parent 5:0 classid 5:1 hfsc sc '
                          + 'rate %fMbit ul rate %fMbit' % ( bw, bw ) ]
            elif use_tbf:
                if latency_ms is None:
                    latency_ms = 15.0 * 8 / bw
                cmds += [ '%s qdisc add dev %s root handle 5: tbf ' +
                          'rate %fMbit burst 15000 latency %fms' %
                          ( bw, latency_ms ) ]
            else:
                cmds += [ '%s qdisc add dev %s root handle 5:0 htb default 1',
                          '%s class add dev %s parent 5:0 classid 5:1 htb ' +
                          'rate %fMbit burst 15k' % bw ]
            parent = ' parent 5:1 '

            # ECN or RED
            if enable_ecn:
                cmds += [ '%s qdisc add dev %s' + parent +
                          'handle 6: red limit 1000000 ' +
                          'min 30000 max 35000 avpkt 1500 ' +
                          'burst 20 ' +
                          'bandwidth %fmbit probability 1 ecn' % bw ]
                parent = ' parent 6: '
            elif enable_red:
                cmds += [ '%s qdisc add dev %s' + parent +
                          'handle 6: red limit 1000000 ' +
                          'min 30000 max 35000 avpkt 1500 ' +
                          'burst 20 ' +
                          'bandwidth %fmbit probability 1' % bw ]
                parent = ' parent 6: '
        return cmds, parent

    @staticmethod
    def delayCmds( parent, delay=None, jitter=None,
                   loss=None, max_queue_size=None ):
        "Internal method: return tc commands for delay and loss"
        cmds = []
        # if delay and float(delay) < 0:
        #     error( 'Negative delay', delay, '\n' )
        # elif jitter and float(jitter) < 0:
        #     error( 'Negative jitter', jitter, '\n' )
        # elif loss and ( float(loss) < 0 or float(loss) > 100 ):
        if loss and ( loss < 0 or loss > 100 ):
            error( 'Bad loss percentage', loss, '%%\n' )
        else:
            # Delay/jitter/loss/max queue size
            netemargs = '%s%s%s%s' % (
                'delay %s ' % delay if delay is not None else '',
                '%s ' % jitter if jitter is not None else '',
                'loss %.5f ' % loss if (loss is not None and loss > 0) else '',
                'limit %d' % max_queue_size if max_queue_size is not None
                else '' )
            if netemargs:
                cmds = [ '%s qdisc add dev %s ' + parent +
                         ' handle 10: netem ' +
                         netemargs ]
                parent = ' parent 10:1 '
        return cmds, parent

    def tc( self, cmd, tc='tc' ):
        "Execute tc command for our interface"
        c = cmd % (tc, self)  # Add in tc command and our name
        debug(" *** executing command: %s\n" % c)
        return self.cmd( c )

    def config( self, bw=None, delay=None, jitter=None, loss=None,
                gro=False, txo=True, rxo=True,
                speedup=0, use_hfsc=False, use_tbf=False,
                latency_ms=None, enable_ecn=False, enable_red=False,
                max_queue_size=None, **params ):
        """Configure the port and set its properties.
           bw: bandwidth in b/s (e.g. '10m')
           delay: transmit delay (e.g. '1ms' )
           jitter: jitter (e.g. '1ms')
           loss: loss (e.g. '1%' )
           gro: enable GRO (False)
           txo: enable transmit checksum offload (True)
           rxo: enable receive checksum offload (True)
           speedup: experimental switch-side bw option
           use_hfsc: use HFSC scheduling
           use_tbf: use TBF scheduling
           latency_ms: TBF latency parameter
           enable_ecn: enable ECN (False)
           enable_red: enable RED (False)
           max_queue_size: queue limit parameter for netem"""

        # Support old names for parameters
        gro = not params.pop( 'disable_gro', not gro )

        result = Intf.config( self, **params)

        def on( isOn ):
            "Helper method: bool -> 'on'/'off'"
            return 'on' if isOn else 'off'

        # Set offload parameters with ethool
        self.cmd( 'ethtool -K', self,
                  'gro', on( gro ),
                  'tx', on( txo ),
                  'rx', on( rxo ) )

        # Optimization: return if nothing else to configure
        # Question: what happens if we want to reset things?
        if ( bw is None and not delay and not loss
             and max_queue_size is None ):
            return

        # Clear existing configuration
        tcoutput = self.tc( '%s qdisc show dev %s' )
        if "priomap" not in tcoutput and "noqueue" not in tcoutput:
            cmds = [ '%s qdisc del dev %s root' ]
        else:
            cmds = []

        # Bandwidth limits via various methods
        bwcmds, parent = self.bwCmds( bw=bw, speedup=speedup,
                                      use_hfsc=use_hfsc, use_tbf=use_tbf,
                                      latency_ms=latency_ms,
                                      enable_ecn=enable_ecn,
                                      enable_red=enable_red )
        cmds += bwcmds

        # Delay/jitter/loss/max_queue_size using netem
        delaycmds, parent = self.delayCmds( delay=delay, jitter=jitter,
                                            loss=loss,
                                            max_queue_size=max_queue_size,
                                            parent=parent )
        cmds += delaycmds

        # Ugly but functional: display configuration info
        stuff = ( ( [ '%.2fMbit' % bw ] if bw is not None else [] ) +
                  ( [ '%s delay' % delay ] if delay is not None else [] ) +
                  ( [ '%s jitter' % jitter ] if jitter is not None else [] ) +
                  ( ['%.5f%% loss' % loss ] if loss is not None else [] ) +
                  ( [ 'ECN' ] if enable_ecn else [ 'RED' ]
                    if enable_red else [] ) )
        info( '(' + ' '.join( stuff ) + ') ' )

        # Execute all the commands in our node
        debug("at map stage w/cmds: %s\n" % cmds)
        tcoutputs = [ self.tc(cmd) for cmd in cmds ]
        for output in tcoutputs:
            if output != '' and output != 'RTNETLINK answers: No such file or directory\r\n':
                error( "*** Error: %s" % output )
        debug( "cmds:", cmds, '\n' )
        debug( "outputs:", tcoutputs, '\n' )
        result[ 'tcoutputs'] = tcoutputs
        result[ 'parent' ] = parent

        return result


class Link( object ):

    """A basic link is just a veth pair.
       Other types of links could be tunnels, link emulators, etc.."""

    # pylint: disable=too-many-branches
    def __init__( self, node1, node2, port1=None, port2=None,
                  intfName1=None, intfName2=None, addr1=None, addr2=None,
                  intf=Intf, cls1=None, cls2=None, params1=None,
                  params2=None, fast=True, **params ):
        """Create veth link to another node, making two new interfaces.
           node1: first node
           node2: second node
           port1: node1 port number (optional)
           port2: node2 port number (optional)
           intf: default interface class/constructor
           cls1, cls2: optional interface-specific constructors
           intfName1: node1 interface name (optional)
           intfName2: node2  interface name (optional)
           params1: parameters for interface 1 (optional)
           params2: parameters for interface 2 (optional)
           **params: additional parameters for both interfaces"""

        # This is a bit awkward; it seems that having everything in
        # params is more orthogonal, but being able to specify
        # in-line arguments is more convenient! So we support both.
        params1 = dict( params1 ) if params1 else {}
        params2 = dict( params2 ) if params2 else {}
        if port1 is not None:
            params1[ 'port' ] = port1
        if port2 is not None:
            params2[ 'port' ] = port2
        if 'port' not in params1:
            params1[ 'port' ] = node1.newPort()
        if 'port' not in params2:
            params2[ 'port' ] = node2.newPort()
        if not intfName1:
            intfName1 = self.intfName( node1, params1[ 'port' ] )
        if not intfName2:
            intfName2 = self.intfName( node2, params2[ 'port' ] )

        # Update with remaining parameter list
        params1.update( params )
        params2.update( params )

        self.fast = fast
        if fast:
            params1.setdefault( 'moveIntfFn', self._ignore )
            params2.setdefault( 'moveIntfFn', self._ignore )
            self.makeIntfPair( intfName1, intfName2, addr1, addr2,
                               node1, node2, deleteIntfs=False )
        else:
            self.makeIntfPair( intfName1, intfName2, addr1, addr2 )

        if not cls1:
            cls1 = intf
        if not cls2:
            cls2 = intf

        intf1 = cls1( name=intfName1, node=node1,
                      link=self, mac=addr1, **params1  )
        intf2 = cls2( name=intfName2, node=node2,
                      link=self, mac=addr2, **params2 )

        # All we are is dust in the wind, and our two interfaces
        self.intf1, self.intf2 = intf1, intf2

    # pylint: enable=too-many-branches

    @staticmethod
    def _ignore( *args, **kwargs ):
        "Ignore any arguments"
        pass

    def intfName( self, node, n ):
        "Construct a canonical interface name node-ethN for interface n."
        # Leave this as an instance method for now
        assert self
        return node.name + '-eth' + repr( n )

    @classmethod
    def makeIntfPair( cls, intfname1, intfname2, addr1=None, addr2=None,
                      node1=None, node2=None, deleteIntfs=True ):
        """Create pair of interfaces
           intfname1: name for interface 1
           intfname2: name for interface 2
           addr1: MAC address for interface 1 (optional)
           addr2: MAC address for interface 2 (optional)
           node1: home node for interface 1 (optional)
           node2: home node for interface 2 (optional)
           (override this method [and possibly delete()]
           to change link type)"""
        # Leave this as a class method for now
        assert cls
        return makeIntfPair( intfname1, intfname2, addr1, addr2, node1, node2,
                             deleteIntfs=deleteIntfs )

    def delete( self ):
        "Delete this link"
        self.intf1.delete()
        self.intf1 = None
        self.intf2.delete()
        self.intf2 = None

    def stop( self ):
        "Override to stop and clean up link as needed"
        self.delete()

    def status( self ):
        "Return link status as a string"
        return "(%s %s)" % ( self.intf1.status(), self.intf2.status() )

    def __str__( self ):
        return '%s<->%s' % ( self.intf1, self.intf2 )


class OVSIntf( Intf ):
    "Patch interface on an OVSSwitch"

    def ifconfig( self, *args ):
        cmd = ' '.join( args )
        if cmd == 'up':
            # OVSIntf is always up
            return
        else:
            raise Exception( 'OVSIntf cannot do ifconfig ' + cmd )


class OVSLink( Link ):
    """Link that makes patch links between OVSSwitches
       Warning: in testing we have found that no more
       than ~64 OVS patch links should be used in row."""

    def __init__( self, node1, node2, **kwargs ):
        "See Link.__init__() for options"
        from containernet.node import OVSSwitch
        self.isPatchLink = False
        if ( isinstance( node1, OVSSwitch ) and
             isinstance( node2, OVSSwitch ) ):
            self.isPatchLink = True
            kwargs.update( cls1=OVSIntf, cls2=OVSIntf )
        Link.__init__( self, node1, node2, **kwargs )

    def makeIntfPair( self, *args, **kwargs ):
        "Usually delegated to OVSSwitch"
        if self.isPatchLink:
            return None, None
        else:
            return Link.makeIntfPair( *args, **kwargs )


class TCLink( Link ):
    "Link with TC interfaces"
    def __init__( self, *args, **kwargs):
        kwargs.setdefault( 'cls1', TCIntf )
        kwargs.setdefault( 'cls2', TCIntf )
        Link.__init__( self, *args, **kwargs)



#
# Copyright (C) 2022 Alexandre Mitsuru Kaihara
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


from subprocess import Popen
from configparser import ConfigParser
from datetime import datetime
import subprocess
import socket
import platform
import urllib3
import sys
	
# Configure backup server 
def configBackupServer(parser):
	
	# Read ips from backup servers 	
	backupIP = parser.get("backup", "ip")
			
	# Mount netstorage 
	try:
		cmd = "mount -t cifs -o username=mininet,password=mininet //" + backupIP + "/backup /home/debian/backup"
		subprocess.check_call(cmd, shell=True)
	except Exception as e:
		with open("/home/debian/log.txt", "a") as file:
			file.write(datetime.now().strftime("%y%m%d-%H%M%S") + " | Fehler beim Mount des Backup-Servers | " + str(e) + "\n")

# Init 
def main():
	# Init parser 
	parser = ConfigParser()
	
	# Open ServerConfig file with parser 
	parser.read("/home/debian/serverconfig.ini")
	
	# Configure Backup strategy 
	configBackupServer(parser)

if __name__ == "__main__":
	main()
